@@ -16,13 +16,17 @@ This file is part of Fwknop2.
  */
 package biz.incomsystems.fwknop2;
 
+import android.Manifest;
 import android.app.Activity;
 import android.app.AlertDialog;
 import android.content.Context;
 import android.content.DialogInterface;
 import android.content.pm.PackageManager;
+import android.os.Build;
 import android.os.Bundle;
+import android.support.v4.app.ActivityCompat;
 import android.support.v4.app.Fragment;
+import android.support.v4.content.ContextCompat;
 import android.text.InputType;
 import android.util.Log;
 import android.view.Gravity;
@@ -100,6 +104,7 @@ public class ConfigDetailFragment extends Fragment {
     CheckBox chkb64hmac ;
     CheckBox chkblegacy;
     CheckBox chkbrandom;
+    CheckBox chkbKeep_Open;
 
     /**
      * The fragment argument representing the item ID that this fragment
@@ -159,7 +164,7 @@ public class ConfigDetailFragment extends Fragment {
         } else if (id == R.id.qr_code) {
             try {
                 IntentIntegrator.forSupportFragment(this).setDesiredBarcodeFormats(IntentIntegrator.QR_CODE_TYPES).initiateScan();
-            } catch (Exception e) { // This is where the play store is called if the app is not installed
+            } catch (Throwable e) { // This is where the play store is called if the app is not installed
 
             }
         } else if (id == R.id.save) {
@@ -173,6 +178,7 @@ public class ConfigDetailFragment extends Fragment {
             if (configtype.equalsIgnoreCase("Open Port")) {//messagetype = configtype
                 config.PORTS = txt_ports.getText().toString();
                 config.SERVER_TIMEOUT = txt_server_time.getText().toString();
+                config.KEEP_OPEN = chkbKeep_Open.isChecked();
                 config.SERVER_CMD = "";
                 config.NAT_PORT = "";
                 config.NAT_IP = "";
@@ -183,6 +189,7 @@ public class ConfigDetailFragment extends Fragment {
                 config.NAT_PORT = txt_nat_port.getText().toString();
                 config.PORTS = txt_ports.getText().toString();
                 config.SERVER_TIMEOUT = txt_server_time.getText().toString();
+                config.KEEP_OPEN = chkbKeep_Open.isChecked();
                 config.SERVER_CMD = "";
             }
             if (configtype.equalsIgnoreCase("Local Nat Access")) {
@@ -190,12 +197,14 @@ public class ConfigDetailFragment extends Fragment {
                 config.NAT_PORT = txt_nat_port.getText().toString();
                 config.PORTS = txt_ports.getText().toString();
                 config.SERVER_TIMEOUT = txt_server_time.getText().toString();
+                config.KEEP_OPEN = chkbKeep_Open.isChecked();
                 config.SERVER_CMD = "";
             }
             if (configtype.equalsIgnoreCase("Server Command")) {
                 config.SERVER_CMD = txt_server_cmd.getText().toString();
                 config.PORTS = "";
                 config.SERVER_TIMEOUT = "";
+                config.KEEP_OPEN = false;
                 config.NAT_PORT = "";
                 config.NAT_IP = "";
             }
@@ -343,9 +352,23 @@ public class ConfigDetailFragment extends Fragment {
     @Override  //This is all the setup stuff for this fragment.
     public View onCreateView(LayoutInflater inflater, ViewGroup container,
                              Bundle savedInstanceState) {
+
+        Thread.setDefaultUncaughtExceptionHandler(
+                new Thread.UncaughtExceptionHandler() {
+                    @Override public void uncaughtException(Thread t, Throwable e) {
+                        Log.e("fwknop2", t.getName()+": "+e);
+                        //MyWorker worker = new MyWorker();
+                        //worker.start();
+                    }
+                });
+
         View rootView = inflater.inflate(R.layout.fragment_config_detail, container, false);
         active_Nick = getArguments().getString("item_id");
-        myJuice = new PluginContract();
+        try {
+            myJuice = new PluginContract();
+        } catch (Throwable e) {
+            Log.e("fwknop2", "Juicessh exception");
+        }
 
         //Handlers for the input fields
         txt_NickName = (TextView) rootView.findViewById(R.id.NickName);
@@ -375,6 +398,7 @@ public class ConfigDetailFragment extends Fragment {
         chkblegacy = (CheckBox) rootView.findViewById(R.id.chkblegacy);
         chkbrandom = (CheckBox) rootView.findViewById(R.id.chkbrandom);
         spn_allowip = (Spinner) rootView.findViewById(R.id.allowip);
+        chkbKeep_Open = (CheckBox) rootView.findViewById(R.id.chkbKeep_open);
         ArrayAdapter<CharSequence> adapter = ArrayAdapter.createFromResource(getActivity(),
                 R.array.spinner_options, android.R.layout.simple_spinner_item);
         adapter.setDropDownViewResource(android.R.layout.simple_spinner_dropdown_item);
@@ -436,8 +460,20 @@ public class ConfigDetailFragment extends Fragment {
         spn_juice = (Spinner) rootView.findViewById(R.id.juice_spn);
 
         if (juiceInstalled) {
-            juice_adapt = new ConnectionSpinnerAdapter(getActivity());
-            spn_juice.setAdapter(juice_adapt);
+            int permissionCheck = ContextCompat.checkSelfPermission(getActivity(),
+                    "com.sonelli.juicessh.api.v1.permission.READ_CONNECTIONS");
+            if (permissionCheck == PackageManager.PERMISSION_DENIED) {
+
+                ActivityCompat.requestPermissions(getActivity(),
+                        new String[]{"com.sonelli.juicessh.api.v1.permission.READ_CONNECTIONS", "com.sonelli.juicessh.api.v1.permission.OPEN_SESSIONS"},
+                        0);
+            }
+            try {
+                juice_adapt = new ConnectionSpinnerAdapter(getActivity());
+                spn_juice.setAdapter(juice_adapt);
+            } catch (Throwable ex) {
+                Log.e("fwknop2", "Juicessh error");
+            }
         } else {
             list.remove(3);
             adapter_ssh.notifyDataSetChanged();
@@ -461,32 +497,39 @@ public class ConfigDetailFragment extends Fragment {
                     spn_juice.setVisibility(View.GONE);
                 } else if (pos == 3) {
                     if (getActivity().checkCallingOrSelfPermission("com.sonelli.juicessh.api.v1.permission.READ_CONNECTIONS") == PackageManager.PERMISSION_GRANTED) {
+                        try {
+                            lay_sshcmd.setVisibility(View.GONE);
+                            lay_ovpncmd.setVisibility(View.GONE);
 
-                        lay_sshcmd.setVisibility(View.GONE);
-                        lay_ovpncmd.setVisibility(View.GONE);
+                            if (connectionListLoader == null) {
 
-                        if (connectionListLoader == null) {
-                            connectionListLoader = new ConnectionListLoader(getActivity(), juice_adapt);
-                            connectionListLoader.setOnLoadedListener(new ConnectionListLoader.OnLoadedListener() {
-                                @Override
-                                public void onLoaded() {  // This is so ugly...
-                                    spn_juice.setVisibility(View.VISIBLE);
-                                    if (config.SSH_CMD.contains("juice:") && spn_juice.getCount() > 0) {
-                                        for (int n = 0; n < spn_juice.getCount(); n++) {
-                                            if (config.SSH_CMD.contains(juice_adapt.getConnectionName(n))) {
-                                                spn_juice.setSelection(n);
+                                connectionListLoader = new ConnectionListLoader(getActivity(), juice_adapt);
+                                connectionListLoader.setOnLoadedListener(new ConnectionListLoader.OnLoadedListener() {
+                                    @Override
+                                    public void onLoaded() {  // This is so ugly...
+                                        spn_juice.setVisibility(View.VISIBLE);
+                                        if (config.SSH_CMD.contains("juice:") && spn_juice.getCount() > 0) {
+                                            for (int n = 0; n < spn_juice.getCount(); n++) {
+                                                if (config.SSH_CMD.contains(juice_adapt.getConnectionName(n))) {
+                                                    spn_juice.setSelection(n);
+                                                }
                                             }
                                         }
                                     }
-                                }
-                            });
-                            getActivity().getSupportLoaderManager().initLoader(0, null, connectionListLoader);
-                        } else {
-                            getActivity().getSupportLoaderManager().restartLoader(0, null, connectionListLoader);
+                                });
+                                getActivity().getSupportLoaderManager().initLoader(0, null, connectionListLoader);
+                            } else {
+                                getActivity().getSupportLoaderManager().restartLoader(0, null, connectionListLoader);
+                            }
+                        } catch (Throwable e) {
+                            Log.e("fwknop2", "Juicessh error");
                         }
                     } else {
                         Context context = getActivity();
                         CharSequence text = getText(R.string.juice_permissions);
+                        if(Build.VERSION.SDK_INT < 23) {
+                            text = getText(R.string.juice_permissions_reinstall);
+                        }
                         int duration = Toast.LENGTH_LONG;
                         Toast toast = Toast.makeText(context, text, duration);
                         toast.setGravity(Gravity.CENTER, 0, 0);
@@ -517,6 +560,7 @@ public class ConfigDetailFragment extends Fragment {
                     configtype = "Open Port";
                     lay_AccessPort.setVisibility(View.VISIBLE);
                     lay_fwTimeout.setVisibility(View.VISIBLE);
+                    chkbKeep_Open.setVisibility(View.VISIBLE);
                     lay_natIP.setVisibility(View.GONE);
                     lay_natport.setVisibility(View.GONE);
                     lay_serverCMD.setVisibility(View.GONE);
@@ -527,6 +571,7 @@ public class ConfigDetailFragment extends Fragment {
                     configtype = "Nat Access";
                     lay_AccessPort.setVisibility(View.VISIBLE);
                     lay_fwTimeout.setVisibility(View.VISIBLE);
+                    chkbKeep_Open.setVisibility(View.VISIBLE);
                     lay_natIP.setVisibility(View.VISIBLE);
                     lay_natport.setVisibility(View.VISIBLE);
                     lay_serverCMD.setVisibility(View.GONE);
@@ -535,6 +580,7 @@ public class ConfigDetailFragment extends Fragment {
                     configtype = "Local Nat Access";
                     lay_AccessPort.setVisibility(View.VISIBLE);
                     lay_fwTimeout.setVisibility(View.VISIBLE);
+                    chkbKeep_Open.setVisibility(View.VISIBLE);
                     lay_natIP.setVisibility(View.GONE);
                     lay_natport.setVisibility(View.VISIBLE);
                     lay_serverCMD.setVisibility(View.GONE);
@@ -544,6 +590,7 @@ public class ConfigDetailFragment extends Fragment {
                     configtype = "Server Command";
                     lay_AccessPort.setVisibility(View.GONE);
                     lay_fwTimeout.setVisibility(View.GONE);
+                    chkbKeep_Open.setVisibility(View.GONE);
                     lay_natIP.setVisibility(View.GONE);
                     lay_natport.setVisibility(View.GONE);
                     lay_serverCMD.setVisibility(View.VISIBLE);
@@ -585,6 +632,7 @@ public class ConfigDetailFragment extends Fragment {
                 chkbrandom.setChecked(true);
             }
             txt_server_time.setText(config.SERVER_TIMEOUT);
+            chkbKeep_Open.setChecked(config.KEEP_OPEN);
             txt_KEY.setText(config.KEY);
             if (config.KEY_BASE64) {
                 chkb64key.setChecked(true);
