@@ -25,6 +25,7 @@ import android.content.Intent;
 import android.content.SharedPreferences;
 import android.net.Uri;
 import android.os.AsyncTask;
+import android.os.Build;
 import android.text.InputType;
 import android.util.Log;
 import android.widget.EditText;
@@ -38,8 +39,14 @@ import com.sonelli.juicessh.pluginlibrary.listeners.OnClientStartedListener;
 import com.sonelli.juicessh.pluginlibrary.listeners.OnSessionFinishedListener;
 import com.sonelli.juicessh.pluginlibrary.listeners.OnSessionStartedListener;
 
-import org.xbill.DNS.*;
 import org.apache.commons.validator.routines.InetAddressValidator;
+import org.xbill.DNS.ARecord;
+import org.xbill.DNS.Lookup;
+import org.xbill.DNS.Record;
+import org.xbill.DNS.Resolver;
+import org.xbill.DNS.SimpleResolver;
+import org.xbill.DNS.Type;
+
 import java.io.BufferedReader;
 import java.io.InputStreamReader;
 import java.io.OutputStream;
@@ -62,6 +69,8 @@ public class SendSPA implements OnSessionStartedListener, OnSessionFinishedListe
     Boolean ready;
     public PluginClient client;
     public boolean isConnected = false;
+    public boolean reKnock;
+    private SharedPreferences prefs;
 
     public native String sendSPAPacket();
 
@@ -119,16 +128,19 @@ public class SendSPA implements OnSessionStartedListener, OnSessionFinishedListe
     @Override
     public void onSessionFinished() {
     }
-
+    public String resend() {
+        final getExternalIP task = new getExternalIP(ourAct);
+        task.execute();
+        return "Success";
+    }
     public int send(String nick, final Activity newAct) {
         ourAct = newAct;
+        prefs = ourAct.getSharedPreferences("MyPreferences", Context.MODE_PRIVATE);
         final InetAddressValidator ipValidate = new InetAddressValidator();
         final IntentIntegrator getQR = new IntentIntegrator(ourAct);
         mydb = new DBHelper(ourAct);
         config = new Config();
         config = mydb.getConfig(nick);
-        //Cursor CurrentIndex = mydb.getData(nick);
-        //CurrentIndex.moveToFirst();
         mydb.close();
 
         //These variables are the ones that the jni pulls settings from.
@@ -169,7 +181,7 @@ public class SendSPA implements OnSessionStartedListener, OnSessionFinishedListe
         final AlertDialog.Builder IPPrompt = new AlertDialog.Builder(ourAct);
         IPPrompt.setTitle("Source IP");
         final EditText input = new EditText(ourAct);
-        input.setInputType(InputType.TYPE_CLASS_TEXT);
+        input.setInputType(InputType.TYPE_CLASS_NUMBER);
         IPPrompt.setView(input);
         IPPrompt.setPositiveButton(ourAct.getResources().getText(R.string.ok), new DialogInterface.OnClickListener() {
             public void onClick(DialogInterface dialog, int whichButton) {
@@ -250,15 +262,17 @@ public class SendSPA implements OnSessionStartedListener, OnSessionFinishedListe
         @Override
         protected void onPreExecute() {
             super.onPreExecute();
-            if(!mActivity.getLocalClassName().equals("biz.incomsystems.fwknop2.NfcKnockActivity")) {
-                pdLoading = new ProgressDialog(mActivity);
-                pdLoading.setMessage("\t" + mActivity.getResources().getText(R.string.sending));
-                pdLoading.show();
+            if(!reKnock) {
+                if (!mActivity.getLocalClassName().equals("biz.incomsystems.fwknop2.NfcKnockActivity")) {
+                    pdLoading = new ProgressDialog(mActivity);
+                    pdLoading.setMessage("\t" + mActivity.getResources().getText(R.string.sending));
+                    pdLoading.show();
+                }
             }
         }
 
         @Override
-        protected String doInBackground(Void... params) {
+        public String doInBackground(Void... params) {
             InetAddressValidator ipValidate = new InetAddressValidator();
 
             try {
@@ -273,7 +287,7 @@ public class SendSPA implements OnSessionStartedListener, OnSessionFinishedListe
 
 
 
-                SharedPreferences prefs = mActivity.getSharedPreferences("MyPreferences", Context.MODE_PRIVATE);
+                //SharedPreferences prefs = mActivity.getSharedPreferences("MyPreferences", Context.MODE_PRIVATE);
                 if (prefs.getBoolean("EnableDns", true)) {
                     try {
                         Resolver resolver = new SimpleResolver("208.67.222.222");
@@ -366,16 +380,20 @@ public class SendSPA implements OnSessionStartedListener, OnSessionFinishedListe
                         conn.getResponseCode();
                         conn.disconnect();
                     }
+                    Thread.sleep(1000);
                 } catch (Exception ex) {
                     return ex.toString();
                 }
-                return mActivity.getResources().getText(R.string.success).toString();
-            } else return mActivity.getResources().getText(R.string.spa_failure).toString();//"Failure generating SPA data";
+
+                return "Success";
+            } else return "Failure generating SPA data"; //mActivity.getResources().getText(R.string.spa_failure).toString();//
         }
 
         @Override
         protected void onPostExecute(String result) {
             super.onPostExecute(result);
+            if (reKnock)
+                return;
             if(!mActivity.getLocalClassName().equals("biz.incomsystems.fwknop2.NfcKnockActivity"))
                 pdLoading.dismiss();
 
@@ -449,6 +467,34 @@ public class SendSPA implements OnSessionStartedListener, OnSessionFinishedListe
                         alertDialog.show();
                     }
                 }
+                    if ((prefs.getBoolean("EnableNotification", true)) && config.SERVER_CMD.equalsIgnoreCase("") && Build.VERSION.SDK_INT > 15) {
+                        Intent newtimer = new Intent(mActivity, knockCountdownTimer.class); //knockCountdownTimer ourtimer = new knockCountdownTimer();
+                        newtimer.putExtra("timeout", config.SERVER_TIMEOUT);
+                        newtimer.putExtra("nickname", config.NICK_NAME);
+                        if (config.KEEP_OPEN) {
+                            newtimer.putExtra("keep_open", true);
+
+                            newtimer.putExtra("access_str", access_str);
+                            newtimer.putExtra("allowip_str", allowip_str);
+                            newtimer.putExtra("passwd_str", passwd_str);
+                            newtimer.putExtra("passwd_b64", passwd_b64);
+                            newtimer.putExtra("hmac_str", hmac_str);
+                            newtimer.putExtra("hmac_b64", hmac_b64);
+                            newtimer.putExtra("fw_timeout_str", fw_timeout_str);
+                            newtimer.putExtra("nat_ip_str", nat_ip_str);
+                            newtimer.putExtra("nat_port_str", nat_port_str);
+                            newtimer.putExtra("nat_access_str", nat_access_str);
+                            newtimer.putExtra("nat_local", nat_local);
+                            newtimer.putExtra("server_cmd_str", server_cmd_str);
+                            newtimer.putExtra("legacy", legacy);
+                            newtimer.putExtra("digest_type", digest_type);
+                            newtimer.putExtra("hmac_type", hmac_type);
+                        }
+
+
+                        mActivity.startService(newtimer);
+                    }
+
             } else {
                 AlertDialog alertDialog = new AlertDialog.Builder(mActivity).create();
                 alertDialog.setTitle("Error");
